#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# Date          : 2026-02-09
# Author        : Lancelot PINCET
# GitHub        : https://github.com/LancelotPincet
# Library       : arrLP
# Module        : image

"""
Image array functions of base shape (y, x).
"""



# %% Libraries




# %% Function
def image(**kwargs) :
    '''
    Image array functions of base shape (y, x).
    
    Parameters
    ----------
    a : int or float
        TODO.

    Returns
    -------
    b : int or float
        TODO.

    Raises
    ------
    TypeError
        TODO.

    Examples
    --------
    >>> from arrlp import image
    ...
    >>> image() # TODO
    '''

    return None



# %% Test function run
if __name__ == "__main__":
    from corelp import test
    test(__file__)